#include <Adafruit_NeoPixel.h>
#include <Wire.h> //I2C Arduino Bibliothek

#define address 0x1E //0011110b, I2C 7Bit Adresse des HMC5883 Chips
#define PIN 8
 
int x; //X Achse
int y; //Y Achse
int z; //Z Achse

int numPixels = 12;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(numPixels, PIN, NEO_GRB + NEO_KHZ800);

uint32_t red = strip.Color(255, 0, 0); //RGB Farbe Rot
uint32_t green = strip.Color(0, 255, 0); //RGB Farbe Grün

void setup() {
  strip.begin();
  strip.setBrightness(5); //die Helligkeit setzen 0 dunke -> 255 ganz hell
  drawGreenRing();

  Serial.begin(9600); //Baudrate
  Wire.begin();
  Wire.beginTransmission(address); //Beginnen der Kommunikation über die Adresse, für das setzen der Konfiguration
  Wire.write(0x02); //select mode register
  Wire.write(0x00); //Modus für das fortlaufende lesen der Werte.
  Wire.endTransmission(); //Beenden der Kommunikation.  
}
 
void loop() {  
 drawGreenRing();
 float northDirection = getNorthDirection();
 int northNeoPixel =  calcNorthNeoPixel(northDirection);
 northNeoPixel = (numPixels-1) - northNeoPixel;
 
 if(northNeoPixel<0){
  northNeoPixel = northNeoPixel *(-1);
 }
 
 strip.setPixelColor(northNeoPixel, red); 
 strip.show();
 delay(1000);
}

//Berechnet aus dem Winkel die Position des NeoPixels.
int calcNorthNeoPixel(float northDirection){
  int result = 0;
  int steps = 360 / numPixels;
  if(northDirection > 0 && northDirection < steps){
    result = 0;
  } else {
    result = northDirection / steps;  
  }
  return result;
}

//Setzt alle NeoPixel in die Farbe Grün.
void drawGreenRing(){
  for(uint16_t i=0; i<strip.numPixels(); i++){
    strip.setPixelColor(i, green); 
  }
  strip.show(); //Anzeigen
}

//Liefert den Winkel in Richtung Norden.
float getNorthDirection(){
  float northDegrees = 0;
  Wire.beginTransmission(address); //Beginnen der Kommunikation
  Wire.write(0x03); //select register 3, X MSB register
  Wire.endTransmission(); //Beenden der Kommunikation
  
  //Lesen der Daten vom 6 fach Register (2 Register pro Achse)
  Wire.requestFrom(address, 6);
  if(6<=Wire.available()){
    x = Wire.read()<<8; //X msb
    x |= Wire.read(); //X lsb
    z = Wire.read()<<8; //Z msb
    z |= Wire.read(); //Z lsb
    y = Wire.read()<<8; //Y msb
    y |= Wire.read(); //Y lsb

    float heading = atan2(y, x);
    
    //Je nach Position auf der Erde muss hier ggf. 
    //eine Anpassung der Deklination geschehen.
    float declinationAngle = 0.22;
    heading += declinationAngle;
  
    if(heading < 0){
      heading += 2*PI;
    }
   
    if(heading > 2*PI){
      heading -= 2*PI;
    }
    
    northDegrees = heading * 180/M_PI; 
  }

  return northDegrees;
}

